@@ -8,41 +8,44 @@ import secrets
 import signal
 import threading
 import urllib.parse
-from collections.abc import Iterable
+from collections.abc import Awaitable, Callable, Iterable
 from contextlib import suppress
 from itertools import chain
 from pathlib import Path
 from tempfile import TemporaryDirectory
 from traceback import print_exc
-from typing import Any
+from typing import Any, TypeVar
 
 import aiohttp
+from aiohttp import ClientTimeout
 from creart import it
-from graia.amnesia.builtins.asgi import UvicornASGIService
+from graia.amnesia.builtins.aiohttp import AiohttpClientService
+from graia.amnesia.builtins.asgi import asgitypes
+from graia.amnesia.builtins.asgi.uvicorn import UvicornASGIService, UvicornOptions
 from launart import Launart, Service, any_completed
 from loguru import logger
 from starlette.applications import Starlette
 from starlette.datastructures import FormData as FormData
 from starlette.requests import Request as StarletteRequest
-from starlette.responses import (
-    FileResponse,
-    HTMLResponse,
-    JSONResponse,
-    PlainTextResponse,
-    Response,
-    StreamingResponse,
-)
+from starlette.responses import FileResponse as FileResponse
+from starlette.responses import HTMLResponse as HTMLResponse
+from starlette.responses import JSONResponse as JSONResponse
+from starlette.responses import PlainTextResponse as PlainTextResponse
+from starlette.responses import Response as Response
+from starlette.responses import StreamingResponse as StreamingResponse
 from starlette.routing import Route, WebSocketRoute
 from starlette.staticfiles import StaticFiles
+from starlette.types import ASGIApp
 from starlette.websockets import WebSocket, WebSocketDisconnect
 from yarl import URL
 
 from satori.const import Api
 from satori.model import Event, Meta, ModelBase, Opcode
+from satori.utils import decode
 
 from .. import EventType
 from .adapter import Adapter as Adapter
-from .conection import WebsocketConnection
+from .connection import WebsocketConnection
 from .formdata import parse_content_disposition as parse_content_disposition
 from .model import Provider as Provider
 from .model import Request as Request
@@ -52,10 +55,12 @@ from .route import RouteCall as RouteCall
 from .route import RouterMixin as RouterMixin
 from .utils import Deque
 
+_T_endpoint = TypeVar("_T_endpoint", bound=Callable[[StarletteRequest], Awaitable[Response] | Response])
+_T_ws_endpoint = TypeVar("_T_ws_endpoint", bound=Callable[[WebSocket], Awaitable[None]])
+StarletteResponse = Response
 
-async def _request_handler(
-    action: str, request: StarletteRequest, func: RouteCall, platform: str, self_id: str
-):
+
+async def _request_handler(action: str, request: StarletteRequest, func: RouteCall, platform: str, self_id: str):
     if action == Api.UPLOAD_CREATE.value:
         async with request.form() as form:
             res = await func(
@@ -73,7 +78,7 @@ async def _request_handler(
             Request(
                 request,
                 action,
-                await request.json(),
+                decode(await request.body()),
                 platform=platform,
                 self_id=self_id,
             )
@@ -111,17 +116,21 @@ class Server(Service, RouterMixin):
         version: str = "v1",
         token: str | None = None,
         webhooks: list[WebhookEndpoint] | None = None,
+        uvicorn_options: UvicornOptions | None = None,
+        *,
         stream_threshold: int = 16 * 1024 * 1024,
         stream_chunk_size: int = 64 * 1024,
     ):
         self.connections = []
-        manager = it(Launart)
-        manager.add_component(UvicornASGIService(host, port))
+        self.host = host
+        self.port = port
         self.version = version
         self.path = path
+        self.uvicorn_options = uvicorn_options
         if self.path and not self.path.startswith("/"):
             self.path = f"/{self.path}"
-        self.url_base = f"http://{host}:{port}{self.path}/{version}"
+        if (self.host == "0.0.0.0" or self.host == "::") and not token:
+            raise ValueError("Token is required when the server is exposed to the public network")
         self.token = token
         self._adapters = []
         self.providers = []
@@ -134,8 +143,58 @@ class Server(Service, RouterMixin):
         self.stream_threshold = stream_threshold
         self.stream_chunk_size = stream_chunk_size
         self.resources: dict[str, Path] = {}
+        self.app = Starlette()
+        self.asgi_service = UvicornASGIService(self.host, self.port, options=self.uvicorn_options)
         super().__init__()
 
+    def replace_app(self, app: ASGIApp | asgitypes.ASGI3Application):
+        """替换当前的 Starlette 应用"""
+        self.app = app
+
+    def asgi_route(
+        self,
+        path: str,
+        methods: list[str] | None = None,
+        name: str | None = None,
+        include_in_schema: bool = True,
+    ) -> Callable[[_T_endpoint], _T_endpoint]:
+        """注册一个 ASGI 路由
+
+        Args:
+            path (str): 路由路径
+            methods (list[str], optional): 支持的 HTTP 方法，默认为 None，表示 ["GET"]
+            name (str, optional): 路由名称，默认为 None
+            include_in_schema (bool, optional): 是否包含在 OpenAPI 文档中，默认为 True
+        """
+
+        def wrapper(endpoint: _T_endpoint, /) -> _T_endpoint:
+            self.app.add_route(path, endpoint, methods=methods, name=name, include_in_schema=include_in_schema)
+            return endpoint
+
+        return wrapper
+
+    def asgi_websocket_route(
+        self,
+        path: str,
+        name: str | None = None,
+    ) -> Callable[[_T_ws_endpoint], _T_ws_endpoint]:
+        """注册一个 ASGI WebSocket 路由
+
+        Args:
+            path (str): 路由路径
+            name (str, optional): 路由名称，默认为 None
+        """
+
+        def wrapper(endpoint: _T_ws_endpoint, /) -> _T_ws_endpoint:
+            self.app.add_websocket_route(path, endpoint, name=name)
+            return endpoint
+
+        return wrapper
+
+    @property
+    def url_base(self):
+        return f"http://{self.host}:{self.port}{self.path}/{self.version}"
+
     def apply(self, item: Provider | Router | Adapter):
         if isinstance(item, Adapter):
             item.ensure_server(self)
@@ -152,14 +211,16 @@ class Server(Service, RouterMixin):
         """在指定路径挂载静态文件"""
         self.resources[route_path] = file
 
-    async def event_callback(self, event: Event):
+    async def post(self, event: Event):
         event.sn = self._sequence
         self._event_cache.append(event)
         self._sequence += 1
         for connection in self.connections:
+            if not connection.alive:
+                continue
             try:
                 await connection.send({"op": Opcode.EVENT, "body": event.dump()})
-            except WebSocketDisconnect:
+            except (WebSocketDisconnect, RuntimeError):
                 break
             except Exception as e:
                 print_exc()
@@ -174,6 +235,7 @@ class Server(Service, RouterMixin):
                         "Satori-OpCode": str(Opcode.EVENT.value),
                     },
                     json=event.dump(),
+                    timeout=ClientTimeout(hook.timeout or 300),
                 ) as resp:
                     resp.raise_for_status()
             except Exception as e:
@@ -183,7 +245,7 @@ class Server(Service, RouterMixin):
     async def websocket_server_handler(self, ws: WebSocket):
         await ws.accept()
         connection = WebsocketConnection(ws)
-        identity = await ws.receive_json()
+        identity = decode(await ws.receive_text())
         if not isinstance(identity, dict) or identity.get("op") != Opcode.IDENTIFY:
             return await ws.close(code=3000, reason="Unauthorized")
         body = identity["body"]
@@ -202,7 +264,7 @@ class Server(Service, RouterMixin):
             {"op": Opcode.READY, "body": {"logins": [lo.dump() for lo in logins], "proxy_urls": proxy_urls}}
         )
         self.connections.append(connection)
-        logger.debug(f"New connection: {id(connection)}")
+        logger.debug(f"New connection: {id(connection):x}")
         heartbeat_task = asyncio.create_task(connection.heartbeat())
         close_task = asyncio.create_task(connection.close_signal.wait())
         try:
@@ -219,7 +281,7 @@ class Server(Service, RouterMixin):
             await any_completed(heartbeat_task, close_task)
         finally:
             await connection.connection_closed()
-            logger.debug(f"Connection closed: {id(connection)}")
+            logger.debug(f"Connection closed: {id(connection):x}")
             heartbeat_task.cancel()
             close_task.cancel()
             self.connections.remove(connection)
@@ -231,9 +293,9 @@ class Server(Service, RouterMixin):
         if "X-Platform" not in request.headers and "Satori-Platform" not in request.headers:
             return Response(status_code=401, content="Missing header X-Platform or Satori-Platform")
         platform: str = request.headers.get("X-Platform") or request.headers.get("Satori-Platform")  # type: ignore
-        if "X-Self-ID" not in request.headers and "Satori-Login-ID" not in request.headers:
-            return Response(status_code=401, content="Missing header X-Self-ID or Satori-Login-ID")
-        self_id: str = request.headers.get("X-Self-ID") or request.headers.get("Satori-Login-ID")  # type: ignore
+        if "X-Self-ID" not in request.headers and "Satori-User-ID" not in request.headers:
+            return Response(status_code=401, content="Missing header X-Self-ID or Satori-User-ID")
+        self_id: str = request.headers.get("X-Self-ID") or request.headers.get("Satori-User-ID")  # type: ignore
 
         for _router in self._adapters:
             if action not in _router.routes:
@@ -255,8 +317,7 @@ class Server(Service, RouterMixin):
             resp = await self.fetch_proxy(url, request)
             # if content size > stream_limit, use streaming response
             if (
-                isinstance(resp, (PlainTextResponse, HTMLResponse, JSONResponse))
-                or resp.__class__ is Response
+                isinstance(resp, (PlainTextResponse, HTMLResponse, JSONResponse)) or resp.__class__ is Response
             ) and len(resp.body) > self.stream_threshold:
 
                 async def iter_content(body: bytes):
@@ -287,13 +348,14 @@ class Server(Service, RouterMixin):
                     file = Path(self._tempdir.name) / path[5:]
                     if file.exists():
                         return FileResponse(file)
-                    raise FileNotFoundError(f"{path[5:]} not found")
                 assert request is not None
                 for provider in self.providers:
                     if provider.ensure(platform, self_id):
                         return await provider.handle_internal(
                             Request(request, "internal", {}, platform=platform, self_id=self_id), path
                         )
+                if path.startswith("_tmp"):
+                    raise FileNotFoundError(f"File not found: {path[5:]}")
                 raise NotImplementedError(f"Login with {platform}:{self_id} not found")
             raise TypeError(f"Invalid internal url: {url}")
 
@@ -345,7 +407,7 @@ class Server(Service, RouterMixin):
         return JSONResponse(content=Meta(logins, proxy_urls).dump())
 
     async def webhook_create_handler(self, request: StarletteRequest):
-        body = await request.json()
+        body = decode(await request.body())
         url = body["url"]
         token = body.get("token")
         self.webhooks.append(WebhookEndpoint(url, token))
@@ -365,7 +427,7 @@ class Server(Service, RouterMixin):
         return Response()
 
     async def webhook_delete_handler(self, request: StarletteRequest):
-        body = await request.json()
+        body = decode(await request.body())
         url = body["url"]
         for endpoint in self.webhooks:
             if endpoint.url == url:
@@ -381,9 +443,8 @@ class Server(Service, RouterMixin):
             self.routes[Api.UPLOAD_CREATE.value] = self._default_upload_create_handler
 
         async with self.stage("preparing"):
-            asgi_service = manager.get_component(UvicornASGIService)
-            app = Starlette(
-                routes=[
+            self.app.routes.extend(
+                [
                     *chain.from_iterable(ada.get_routes() for ada in self._adapters),
                     WebSocketRoute(f"{self.path}/{self.version}/events", self.websocket_server_handler),
                     Route(
@@ -414,12 +475,14 @@ class Server(Service, RouterMixin):
                 ]
             )
             for path, file in self.resources.items():
-                app.mount(path, StaticFiles(directory=file.parent, html=file.suffix == ".html"))
-            asgi_service.middleware.mounts[""] = app  # type: ignore
+                self.app.mount(path, StaticFiles(directory=file.parent, html=file.suffix == ".html"))
+            self.asgi_service.middleware.mounts[""] = self.app  # type: ignore
 
         async def event_task(_provider: Provider):
-            async for event in _provider.publisher():
-                await self.event_callback(event)
+            async for event in _provider.publisher():  # type: ignore
+                await self.post(event)
+
+        event_tasks = [event_task(_provider) for _provider in self.providers if hasattr(_provider, "publisher")]
 
         async with self.stage("blocking"):
             proxy_urls = []
@@ -434,17 +497,18 @@ class Server(Service, RouterMixin):
                         "Satori-OpCode": str(Opcode.META.value),
                     },
                     json={"proxy_urls": proxy_urls},
+                    timeout=ClientTimeout(hook.timeout or 300),
                 ) as resp:
                     resp.raise_for_status()
             await any_completed(
                 manager.status.wait_for_sigexit(),
-                *(event_task(provider) for provider in self.providers),
+                *event_tasks,
                 *(_adapter.status.wait_for("blocking-completed") for _adapter in self._adapters),
             )
 
         async with self.stage("cleanup"):
             with suppress(KeyError):
-                del asgi_service.middleware.mounts[""]
+                del self.asgi_service.middleware.mounts[""]
             await self.session.close()
             self._tempdir.cleanup()
 
@@ -457,7 +521,10 @@ class Server(Service, RouterMixin):
     ):
         if manager is None:
             manager = it(Launart)
+        manager.add_component(self.asgi_service)
         manager.add_component(self)
+        with suppress(ValueError):
+            manager.add_component(AiohttpClientService())
         manager.launch_blocking(loop=loop, stop_signal=stop_signal)
 
     async def run_async(
@@ -467,7 +534,10 @@ class Server(Service, RouterMixin):
     ):
         if manager is None:
             manager = it(Launart)
+        manager.add_component(self.asgi_service)
         manager.add_component(self)
+        with suppress(ValueError):
+            manager.add_component(AiohttpClientService())
         handled_signals: dict[signal.Signals, Any] = {}
         launch_task = asyncio.create_task(manager.launch(), name="amnesia-launch")
         signal_handler = functools.partial(manager._on_sys_signal, main_task=launch_task)
